<?php
/**
 * Grimlock_Video_Section_Block Class
 *
 * @author  Themosaurus
 * @package  grimlock-video
 */

if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

/**
 * The class that extends the Section block to add video options
 */
class Grimlock_Video_Section_Block {
	/**
	 * @var string Block base id. Used to generate hook names.
	 */
	protected $id_base;

	/**
	 * Grimlock_Video_Section_Block constructor.
	 *
	 * @param string $id_base ID of the extended block
	 */
	public function __construct( $id_base = 'grimlock_section_block' ) {
		$this->id_base = $id_base;

		// General tab
		add_filter( "{$this->id_base}_general_panel_fields", array( $this, 'add_separator'       ), 109 );
		add_filter( "{$this->id_base}_general_panel_fields", array( $this, 'add_video_url_field' ), 109 );

		// Style tab
		add_filter( "{$this->id_base}_style_panel_fields", array( $this, 'add_separator'                     ), 107 );
		add_filter( "{$this->id_base}_style_panel_fields", array( $this, 'add_background_video_url_field'    ), 107 );
		add_filter( "{$this->id_base}_style_panel_fields", array( $this, 'add_background_video_volume_field' ), 108 );
		add_filter( "{$this->id_base}_style_panel_fields", array( $this, 'add_background_video_loop_field'   ), 109 );

		add_filter( "{$this->id_base}_defaults",       array( $this, 'change_defaults'       ), 10    );
		add_filter( "{$this->id_base}_component_args", array( $this, 'change_component_args' ), 10, 2 );
	}

	/**
	 * Add a text field to set the url of the video
	 *
	 * @param array $fields Array of block fields
	 *
	 * @return array Modified array of block fields
	 */
	public function add_video_url_field( $fields ) {
		$fields[] = Grimlock_Base_Block::text_field( apply_filters( "{$this->id_base}_video_url_field_args", array(
			'name'        => 'video_url',
			'label'       => esc_html__( 'Video URL', 'grimlock-video' ),
			'description' => esc_html__( 'Note that you must also choose a thumbnail for the video to be displayed.', 'grimlock-video' ),
		) ) );

		return $fields;
	}

	/**
	 * Add a text field to set the url of the background video
	 *
	 * @param array $fields Array of block fields
	 *
	 * @return array Modified array of block fields
	 */
	public function add_background_video_url_field( $fields ) {
		$fields[] = Grimlock_Base_Block::text_field( apply_filters( "{$this->id_base}_background_video_url_field_args", array(
			'name'  => 'background_video_url',
			'label' => esc_html__( 'Background Video URL', 'grimlock-video' ),
		) ) );

		return $fields;
	}

	/**
	 * Add a range field to set the volume of the background video
	 *
	 * @param array $fields Array of block fields
	 *
	 * @return array Modified array of block fields
	 */
	public function add_background_video_volume_field( $fields ) {
		$fields[] = Grimlock_Base_Block::range_field( apply_filters( "{$this->id_base}_background_video_volume_field_args", array(
			'name'  => 'background_video_volume',
			'min'   => 0,
			'max'   => 100,
			'step'  => 1,
			'unit'  => '%',
			'label' => esc_html__( 'Background Video Volume', 'grimlock-video' ),
		) ) );

		return $fields;
	}

	/**
	 * Add a checkbox to set whether the background video is looped in the widget
	 *
	 * @param array $fields Array of block fields
	 *
	 * @return array Modified array of block fields
	 */
	public function add_background_video_loop_field( $fields ) {
		$fields[] = Grimlock_Base_Block::toggle_field( apply_filters( "{$this->id_base}_background_video_loop_field_args", array(
			'name'  => 'background_video_loop',
			'label' => esc_html__( 'Loop background video', 'grimlock-video' ),
		) ) );

		return $fields;
	}

	/**
	 * Add a separator to space the fields
	 *
	 * @param array $fields Array of block fields
	 *
	 * @return array Modified array of block fields
	 */
	public function add_separator( $fields ) {
		$fields[] = Grimlock_Base_Block::separator();

		return $fields;
	}

	/**
	 * Change the default settings for the block
	 *
	 * @param array $defaults The default settings for the block.
	 *
	 * @return array The updated default settings for the block.
	 */
	public function change_defaults( $defaults ) {
		return array_merge( $defaults, array(
			'video_url'               => '',
			'background_video_url'    => '',
			'background_video_volume' => 0,
			'background_video_loop'   => true,
		) );
	}

	/**
	 * Change the arguments sent to the component in charge of rendering the block
	 *
	 * @param array $component_args The arguments for the component in charge of rendering the block
	 * @param array $attributes Block attributes
	 *
	 * @return array The updated arguments for the component in charge of rendering the block
	 */
	public function change_component_args( $component_args, $attributes ) {
		$attributes = $this->sanitize_attributes( $attributes );

		$video_section_uniqid = uniqid();
		$component_args['class'][] = "grimlock-video-section--{$video_section_uniqid}";

		$video_property = array(
			'videoURL'           => $attributes['background_video_url'],
			'containment'        => ".grimlock-video-section--{$video_section_uniqid}.grimlock-section",
			'ratio'              => 'auto',
			'optimizeDisplay'    => true,
			'autoPlay'           => true,
			'mute'               => $attributes['background_video_volume'] <= 0,
			'vol'                => $attributes['background_video_volume'],
			'startAt'            => 0,
			'opacity'            => 1,
			'gaTrack'            => false,
			'showControls'       => false,
			'showYTLogo'         => false,
			'loop'               => $attributes['background_video_loop'],
			'remember_last_time' => false,
			'stopMovieOnBlur'    => false,
		);

		return array_merge( $component_args, array(
			'data_attributes' => array(
				'property' => esc_attr( json_encode( $video_property ) ),
			),
			'video_url'       => $attributes['video_url'],
		) );
	}

	/**
	 * Handles sanitizing attributes for the current block instance.
	 *
	 * @param array $new_attributes New attributes for the current block instance
	 *
	 * @return array Attributes to save
	 */
	public function sanitize_attributes( $new_attributes ) {
		$attributes = $new_attributes;

		$attributes['video_url']               = isset( $new_attributes['video_url'] ) ? esc_url( $new_attributes['video_url'] ) : '';
		$attributes['background_video_url']    = isset( $new_attributes['background_video_url'] ) ? esc_url( $new_attributes['background_video_url'] ) : '';
		$attributes['background_video_volume'] = isset( $new_attributes['background_video_volume'] ) ? sanitize_text_field( $new_attributes['background_video_volume'] ) : 0;
		$attributes['background_video_loop']   = filter_var( $new_attributes['background_video_loop'], FILTER_VALIDATE_BOOLEAN );

		return $attributes;
	}
}

return new Grimlock_Video_Section_Block();
