<?php
/**
 * Grimlock_Video_Section_Widget_Fields Class
 *
 * @author  Themosaurus
 * @since   1.0.0
 * @package  grimlock
 */

if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

/**
 * The class that extends WP_Widget to create a section widget.
 *
 * @see WP_Widget
 */
class Grimlock_Video_Section_Widget_Fields extends Grimlock_Base_Widget_Fields {
	/**
	 * Sets up a new Section widget instance.
	 *
	 * @since 1.0.0
	 * @access public
	 */
	public function __construct( $id_base = 'grimlock_section_widget' ) {
		parent::__construct( $id_base );

		// General tab
		add_action( "{$this->id_base}_general_tab", array( $this, 'add_video_url_field'               ), 101, 2 );

		// Style tab
		add_action( "{$this->id_base}_style_tab",   array( $this, 'add_background_video_url_field'    ), 121, 2 );
		add_action( "{$this->id_base}_style_tab",   array( $this, 'add_background_video_volume_field' ), 122, 2 );
		add_action( "{$this->id_base}_style_tab",   array( $this, 'add_background_video_loop_field'   ), 123, 2 );
	}

	/**
	 * Add a text field to set the url of the background video
	 *
	 * @param array $instance
	 * @param WP_Widget $widget
	 * @since 1.0.0
	 */
	public function add_video_url_field( $instance, $widget ) {
		$args = array(
			'id'    => $widget->get_field_id( 'video_url' ),
			'name'  => $widget->get_field_name( 'video_url' ),
			'label' => esc_html__( 'Video URL', 'grimlock-video' ),
			'value' => $instance['video_url'],
		);

		$this->textfield( apply_filters( "{$this->id_base}_video_url_field_args", $args, $instance ) );
	}

	/**
	 * Add a text field to set the url of the background video
	 *
	 * @param array $instance
	 * @param WP_Widget $widget
	 * @since 1.0.0
	 */
	public function add_background_video_url_field( $instance, $widget ) {
		$args = array(
			'id'    => $widget->get_field_id( 'background_video_url' ),
			'name'  => $widget->get_field_name( 'background_video_url' ),
			'label' => esc_html__( 'Background Video URL', 'grimlock-video' ),
			'value' => $instance['background_video_url'],
		);

		$this->textfield( apply_filters( "{$this->id_base}_background_video_url_field_args", $args, $instance ) );
	}

	/**
	 * Add a range field to set the volume of the background video
	 *
	 * @param $instance
	 * @param WP_Widget $widget
	 * @since 1.0
	 */
	public function add_background_video_volume_field( $instance, $widget ) {
		$args = array(
			'id'    => $widget->get_field_id( 'background_video_volume' ),
			'name'  => $widget->get_field_name( 'background_video_volume' ),
			'min'   => 0,
			'max'   => 100,
			'step'  => 1,
			'label' => esc_html__( 'Background Video Volume', 'grimlock-video' ),
			'value' => $instance['background_video_volume'],
		);

		$this->range( apply_filters( "{$this->id_base}_background_video_volume_field_args", $args, $instance ) );
	}

	/**
	 * Add a checkbox to set whether the background video is looped in the widget
	 *
	 * @param array $instance
	 * @param WP_Widget $widget
	 * @since 1.0.0
	 */
	public function add_background_video_loop_field( $instance, $widget ) {
		$args = array(
			'id'    => $widget->get_field_id( 'background_video_loop' ),
			'name'  => $widget->get_field_name( 'background_video_loop' ),
			'label' => esc_html__( 'Loop Background Video', 'grimlock-video' ),
			'value' => $instance['background_video_loop'],
		);

		$this->checkbox( apply_filters( "{$this->id_base}_background_video_loop_field_args", $args, $instance ) );
	}

	/**
	 * Change the default settings for the widget
	 *
	 * @param array $defaults The default settings for the widget.
	 *
	 * @return array The updated default settings for the widget.
	 */
	public function change_defaults( $defaults ) {
		return array_merge( $defaults, array(
			'video_url'               => '',
			'background_video_url'    => '',
			'background_video_volume' => 0,
			'background_video_loop'   => true,
		) );
	}

	/**
	 * Change the arguments sent to the component in charge of rendering the widget
	 *
	 * @param array $component_args The arguments for the component in charge of rendering the widget
	 * @param array $instance Settings for the current widget instance.
	 * @param array $widget_args Display arguments including 'before_title', 'after_title',
	 *                           'before_widget', and 'after_widget'.
	 * @param string $widget_id The id of the current widget instance
	 *
	 * @return array The updated arguments for the component in charge of rendering the widget
	 */
	public function change_component_args( $component_args, $instance, $widget_args, $widget_id ) {
		$video_property = array(
			'videoURL'           => $instance['background_video_url'],
			'containment'        => "#{$widget_id} .section",
			'ratio'              => 'auto',
			'optimizeDisplay'    => true,
			'autoPlay'           => true,
			'mute'               => $instance['background_video_volume'] <= 0,
			'vol'                => $instance['background_video_volume'],
			'startAt'            => 0,
			'opacity'            => 1,
			'gaTrack'            => false,
			'showControls'       => false,
			'showYTLogo'         => false,
			'loop'               => $instance['background_video_loop'],
			'remember_last_time' => false,
			'stopMovieOnBlur'    => false,
		);

		return array_merge( $component_args, array(
			'data_attributes' => array(
				'property' => esc_attr( json_encode( $video_property ) ),
			),
			'video_url'       => $instance['video_url'],
		) );
	}

	/**
	 * Handles sanitizing settings for the current widget instance.
	 *
	 * @since 1.0.0
	 * @access public
	 *
	 * @param array $new_instance New settings for this instance as input by the user via
	 *                            WP_Widget::form().
	 * @param array $old_instance Old settings for this instance.
	 *
	 * @return array Settings to save or bool false to cancel saving.
	 */
	public function sanitize_instance( $new_instance, $old_instance ) {
		$instance = array_merge( $old_instance, $new_instance );

		$instance['video_url']               = isset( $new_instance['video_url'] ) ? esc_url( $new_instance['video_url'] ) : '';
		$instance['background_video_url']    = isset( $new_instance['background_video_url'] ) ? esc_url( $new_instance['background_video_url'] ) : '';
		$instance['background_video_volume'] = isset( $new_instance['background_video_volume'] ) ? intval( $new_instance['background_video_volume'] ) : 0;
		$instance['background_video_loop']   = ! empty( $new_instance['background_video_loop'] );

		return $instance;
	}
}

return new Grimlock_Video_Section_Widget_Fields();
